'#Reference {00020813-0000-0000-C000-000000000046}#1.5#0#C:\Program Files\Microsoft Office\OFFICE11\EXCEL.EXE#Microsoft Excel 11.0 Object Library#Excel
' File: FreebodyLoadsGPVectorsWExcel_07.BAS
' Version: Version 07
' Date: February 6, 2018
' Author: Jonathan Buck, Ryan Blelloch, Emily Jewell, Kevin Cohen, ATA Engineering
' www.ata-plmsoftware.com
' Description: This API calculates the freebody loads (i.e. resultant force and moment components) for a section cut 
'  through a FEM similar to the way the Freebody Postprocessing tool works in Femap v11.0. Freebody loads will be 
'  calculated for each chosen group for all chosen load cases.
'
'Usage Notes
'====================================
'1. To begin with,You must request grid point force output in your Nastran run
'2. Create an arbitrary number of groups to define your section cuts. Each of these groups should consist of the following:
'    a. A ring, strip, slice,or annular segment of elements.
'    b. The nodes on only one side of the elements selected in the previous step. Freebody loads
'       will be calculated using the data on these nodes such that a tensile load will be directed outward from the elements.  
'    c. Optionally, a single coordinate system. Freebody loads will be transformed into this coordinate system if desired.
'Once these steps prepatory steps have been taken, run the analysis, import the results, and run the API
'1. Select the desired group(s). Each group selected will have its own summary output. Note: If a group has nodes but not elements,
'   it will note appear in this list!
'2. Choose desired output sets. Freebody loads will be calculated for each selected output set. 
'3. Lastly, select desired optional settings. 
'     a. For more detail on the effect of "Include Applied Forces," "Include Reaction Forces," and 
'        "Include Multipoint Constraint Forces," see the relevant Femap documentation. These settings are equivalent to the same settings that
'         appear in the Freebody section of the Post Processing Toolbox. 
'     b. You can choose to send results to the Message Window inside Femap or write results out to an Excel spreadsheet.
'     c. The "Keep Freebody Set" option will create a Freebody object title "Dummy_Freebody_Set" in the Post Processing Toolbox that is populated with
'        the settings selected through this API.
'     d. By default, the freebody section loads will be calculated in the basic coordinate system and about the origin of the basic coordinate system. 
'        If a single coordinate system exists in any of your groups, you may use the "Sum about Group Coordinate System Origin" option to use this coordinate
'        system in all the freebody calculations instead of the basic coordinate system. Groups without coordinate systems included will still reference
'        the basic coordinate system. Note: If using this option, be sure that only one coordinate system is included per group.
'
'Copyright (c) <2018> <ATA Engineering, Inc.>
'
'Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
'associated documentation files (the "Software") to use, copy, modify, merge, and/or distribute
'royalty-free copies of the Software, and to permit persons to whom the Software is furnished
'to do so, subject to the following conditions:
'The above copyright notice and this permission notice shall be included in all copies or
'substantial portions of the Software.
'THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
'INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
'FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
'IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
'WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
'OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.       
'
'*************************
'*** Revision History ****
'*************************
'
' Version: Version 00
' Date: June 26, 2014
' Author: Ryan Blelloch, ATA Engineering
' Description: Initial Release
'
' Version: Version 01
' Date: June 26, 2014
' Author: Jonathan Buck, ATA Engineering
' Description: Introduced the CalculateNodalCenter method of the freebody object
' so that the summation point for resultant moments is the average position of
' all nodes in the selected group
'
'Version: Version 02
'Date: June 27, 2014
'Author: Ryan Blelloch, ATA Engineering
'Description: Changed group and output selection to a single form. Form also
'includes options to add applied contributions, reactions, and multipoint
'contributions, use Group coordinate systems if avaliable,  keep the Freebody
'Object after completion, and send results to excel sheet

'Version: Version 03
'Date: July 2, 2014
'Author: Ryan Blelloch
'Description: Added the option to use alternate vectors for the force contributions
'and changed message window to only show results when user is not sending
'results to Excel

'Version: Version 04
'Date: January 5th, 2014
'Author: Jonathan Buck
'Description: Added usage notes from "FreebodyLoadsFromGPForces_ver9.BAS". No change to code.
'
'Version: Version 06
'Date: February 10th, 2015
'Author: Emily Jewell, ATA Engineering 
'Description: Modified output group selection options so that all outputs could be calculated.
'Simple coding error where "legalGroups" needed to be changed to "legalOutputs"
'
'Version: Version 07
'Date: February 6th, 2018
'Author: Kevin Cohen, ATA Engineering
'Description: Changed method of deleting autogenerated sheets to adapt to any excel default number of sheets.
'

Sub Main

	'Connect to model currently open
    Dim App As femap.model
    Set App = feFemap()

    'Holds user-specified groups
    Dim gSet As femap.Set
    Set gSet = App.feSet

    'Used to store indiviual group ID
    Dim gID As Long

    'Holds user-specified output sets
    Dim oSet As femap.Set
    Set oSet = App.feSet

    'Used to store individual output set ID
    Dim oID As Long

    'Stores the number of output sets selected
    Dim numOutputSets As Long

    'Stores the number of output sets that were able to be correctly calculated by CalculateSummation2
    Dim numSuccessOutputs As Long

    'Stores the output set ID's that were correctly calculated by CalculateSummation2
    Dim IDSuccessOutputs As Variant

    'Stores the calculations of the successful output sets in the CalculateSummation2 method
    Dim calculationsOnOutputSets As Variant

	'Stores the location of the average position of all nodes in a group for calculating resultants
	Dim averageNodalCenter As Variant

    'Stores value of Fx
    Dim Fx_Val As Variant
    'Stores value of Fy
    Dim Fy_Val As Variant
    'Stores value of Fz
    Dim Fz_Val As Variant
    'Stores value of Mx
    Dim Mx_Val As Variant
    'Stores value of My
    Dim My_Val As Variant
    'Stores value of Mz
    Dim Mz_Val As Variant

    'Temporarily holds element set of a group for check
    Dim eSet As Set

    'Temporarily holds node set of a group for check
    Dim nSet As Set

    'Temporarily holds coordinate system set of a group for check
    Dim cSet As Set

    'Group object used to loop through and get group info
    Dim groups As femap.Group
    Set groups = App.feGroup

    'Output Sets object used to loop through and output set info
    Dim outputSets As femap.OutputSet
    Set outputSets = App.feOutputSet

    'Freebody object used to create dummy freebody set for analysis
    Dim freebodySet As femap.Freebody
    Set freebodySet = App.feFreebody

    'Freebody ID
    Dim fID As Long

    'Used to store user decisions
    Dim userInput As Variant

    'Used to check if the user selected any groups
    Dim gExist As Boolean
    gExist = False

    'Used to check if all selected groups have elements
    Dim gElemExist As Boolean

    'Used to check if all selected groups have nodes
    Dim gNodeExist As Boolean

    'Used to check if a group has at least one coordinate system
    Dim cSysExist As Boolean

    'Used to check if user wants results sent to new Excel spreadsheet
    Dim toExcel As Boolean
    toExcel = False

    'Used to check if user wants results sent to Femap message board
    Dim toMessageBoard As Boolean
    toMessageBoard = True

    'User chooses whether or not to include applied forces
    Dim appliedForces As Boolean
    appliedForces = False

    'User chooses whether or not to include reaction forces
    Dim reactionForces As Boolean
    reactionForces = False

    'User chooses whether or not to include multipoint constraint forces
    Dim multipointForces As Boolean
    multipointForces = False

    'Use alternate vectors for applied forces
    Dim appliedAltVect As Boolean
    appliedAltVect = False

    'Use alternate vectors for reaction forces
    Dim reactionAltVect As Boolean
    reactionAltVect = False

    'Use alternate vectors for multipoint constraint forces
    Dim multipointAltVect As Boolean
    multipointAltVect = False

    'User chooses whether or not to use group coordinate systems whenever possible
    Dim groupCoordSys As Boolean
    groupCoordSys = False

    'User chooses whether or not to keep the freebody avaliable after the macro is run
    Dim keepFreebody As Boolean
    keepFreebody = False

    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    'Prepares an Excel application object (will not neccesarily be used)
    Dim appExcel As Excel.Application

    'Prepares an Excel workbook object (will not neccesarily be used)
    Dim wbk As Excel.Workbook

    'Prepares an Excel worksheet object (will not neccesarily be used)
    Dim wks As Excel.Worksheet

    'Counts the number of sheets being used
    Dim numSheets As Long
    numSheets = 0

    'Stores the row and column indices for the excel spreadsheets
    Dim row As Long
    Dim col As Long
    row = 1
    col = 1

    '*************************************************************************************************************************************************

    Dim numOfLegalGroups As Long 'Used to index the group array
    numOfLegalGroups  = 0

    Dim numOfOutputSets As Long 'Used to index the output set array
    numOfOutputSets = 0

    Dim legalGroups() As String
    ReDim legalGroups(groups.CountSet) 'Dimensions group array to the maximum possible number of legal groups
    Dim legalOutputs() As String
    ReDim legalOutputs(outputSets.CountSet) 'Dimensions output set array to number of outputs in model

    'Stores the parsed ID character of the original string
    Dim parsedString As String

    'Stores the number of characters the ID in the string is
    Dim IDLengthInString As Long

    'Stores ID of a chosen group or output set
    Dim chosenID As Long

    'Finds all groups with both nodes and elements for user selection
    gSet.AddAll(FT_GROUP)
    gID = gSet.First
    While gID > 0

    	groups.Get(gID)
    	gElemExist = True
    	Set eSet =groups.List(FGR_ELEM)
    	If eSet Is Nothing Then 'Tests to see if any elements exist in the group
    		gElemExist = False
    	End If
    	gNodeExist = True
    	Set nSet = groups.List(FGR_NODE)
    	If nSet Is Nothing Then 'Tests to see if any nodes exist in the group
    		gNodeExist = False
    	End If

    	'If a group has both nodes and elements, its ID and title are stored for user selection
    	If gElemExist = True And gNodeExist = True Then
    		numOfLegalGroups = numOfLegalGroups + 1
    		legalGroups(numOfLegalGroups - 1) = CStr(gID) & ". " & groups.title
    	End If

    	gID = gSet.Next
    Wend

    'Finds all avaliable outputs and stores string titles for user selection
    oSet.AddAll(FT_OUT_CASE)
    oID = oSet.First
    While oID > 0
    	outputSets.Get(oID)
    	numOfOutputSets = numOfOutputSets + 1
    	legalOutputs(numOfOutputSets - 1) = CStr(oID) & ". " & outputSets.title

    	oID = oSet.Next
    Wend

	Begin Dialog UserDialog 870,455,"Freebody Loads from GP Vectors" ' %GRID:10,7,1,1
		MultiListBox 30,42,390,273,legalGroups(),.Group_Choices
		MultiListBox 430,42,400,273,legalOutputs(),.Output_Set_Choices
		OKButton 590,413,120,28
		CancelButton 730,413,110,28
		text 40,21,110,21,"Choose Groups:",.ChooseGroups
		text 450,21,140,21,"Choose Output Sets:",.ChooseOutputSets
		CheckBox 20,364,190,14,"Include Applied Forces",.aL
		CheckBox 20,385,190,14,"Include Reaction Forces",.rL
		CheckBox 550,385,310,14,"Sum about Group Coordinate System Origins",.gCS
		CheckBox 550,364,160,14,"Keep Freebody Set",.kF
		CheckBox 20,406,260,14,"Include MultiPoint Constraint Forces",.mPCF
		CheckBox 280,364,20,14,"CheckBox1",.aAV
		CheckBox 280,385,20,14,"CheckBox2",.rAV
		CheckBox 280,406,20,14,"CheckBox3",.mpcAV
		text 250,322,80,35,"Use Alternate Vectors",.aVText,2
		text 20,343,110,14,"Forces to Include",.forcesText,2
		OptionGroup .sendResults
			OptionButton 360,364,150,14,"Excel",.OptionButtonE
			OptionButton 360,385,150,14,"Message Board",.OptionButtonM
		text 360,343,110,14,"Send Results To",.resultsText,2
		text 550,343,100,14,"Other Options",.oOptionsText,2
	End Dialog
	Dim dlg As UserDialog
	userInput = Dialog(dlg) 'Tests whether the user pressed okay, cancel, or deleted the form

	'If user cancels the form, the macro is exited
	If userInput = FE_FAIL Or userInput = FE_CANCEL Then
		App.feAppMessage(FCM_NORMAL, "User Exited Macro")
		End
	End If

	'Adds all of the groups chosen into the gSet with string parsing
	gSet.Clear
	For i = LBound(dlg.Group_Choices) To UBound(dlg.Group_Choices)
		IDLengthInString = 1
		chosenID = dlg.Group_Choices(i)
		parsedString = Mid(legalGroups(chosenID), 1, 1)
		While parsedString <> "."  'Searches for the period that signals the end of the ID
			IDLengthInString = IDLengthInString + 1
			parsedString = Mid(legalGroups(chosenID), IDLengthInString, 1)
		Wend
		parsedString = Left(legalGroups(chosenID), IDLengthInString - 1)
		If IsNumeric(parsedString) Then
			gSet.Add(Val(parsedString))   'Adds the integer conversion of the parsed string to the gSet
		Else
			App.feAppMessage(FCM_ERROR, "String parsing error in groups. Exiting macro...")
			End  'Macro is exited
		End If
	Next

	'Adds all of the output sets chosen into the oSet with string parsing
	oSet.Clear
	For i = LBound(dlg.Output_Set_Choices) To UBound(dlg.Output_Set_Choices)
		IDLengthInString = 1
		chosenID = dlg.Output_Set_Choices(i)
		parsedString = Mid(legalOutputs(chosenID), IDLengthInString, 1)
		While parsedString <> "."  'Searches for the period that signals the end of the ID
			IDLengthInString = IDLengthInString + 1
			parsedString = Mid(legalOutputs(chosenID), IDLengthInString, 1)
		Wend
		parsedString = Left(legalOutputs(chosenID), IDLengthInString - 1)
		If IsNumeric(parsedString) Then
			oSet.Add(Val(parsedString))  'Adds the integer conversion of the parsed string to the oSet
		Else
			App.feAppMessage(FCM_ERROR, "String parsing error in output sets. Exiting macro...")
			End  'Macro is exited
		End If
	Next

	'User selection of applied forces checkbox
	If dlg.aL = 1 Then
		appliedForces = True
	End If
	'User selection of reaction forces checkbox
	If dlg.rL = 1 Then
		reactionForces = True
	End If
	'User selection of multipoint constraint forces
	If dlg.mPCF = 1 Then
		multipointForces = True
	End If
	'User selection of alternate vectors for applied forces
	If dlg.aAV = 1 Then
		appliedAltVect = True
	End If
	'User selection of alternate vectors for reaction forces
	If dlg.rAV = 1 Then
		reactionAltVect = True
	End If
	'User selection of alternate vectors for multipoint constraint forces
	If dlg.mpcAV = 1 Then
		multipointAltVect = True
	End If
	'User selection of group coordinate system checkbox
	If dlg.gCS = 1 Then
		groupCoordSys = True
	End If
	'User selection of keep freebody checkbox
	If dlg.kF = 1 Then
		keepFreebody = True
	End If

	'User selection of way to show results (Excel or Message Board)
	Select Case dlg.sendResults
		Case 0 'User chooses Excel
			toExcel = True
			toMessageBoard = False
		Case 1 'User chooses message board
			toMessageBoard = True
	End Select


    '*************************************************************************************************************************************************

    '<Creates a new freebody and sets its properties>
    fID = freebodySet.NextEmptyID 'Makes sure it does not override another freebody object
    freebodySet.title = "Dummy_Freebody_Set" 'Sets title in case of error in deletion
    freebodySet.NodalVectorMode(0) = 0 'Sets Nodal Force Vector Display to "Off"
    freebodySet.NodalVectorMode(1) = 0 'Sets Nodal Moment Vector Display to "Off"
    freebodySet.TotalVectorMode(0) = 2 'Sets Total Force Vector Display to "Display Resultant"
    freebodySet.TotalVectorMode(1) = 2 'Sets Total Moment Vector Display to "Display Resultant"
    displayMode = 1 '0 = Freebody, 1 = Interface Load
    If displayMode = 0 Then
		freebodySet.DisplayMode = FBD_DISPLAYMODE_FREEBODY
	End If
    If displayMode = 1 Then
		freebodySet.DisplayMode = FBD_DISPLAYMODE_INTERFACE 'Sets Display Mode to "Interface Load"
	End If
    freebodySet.SumContributions(0) = appliedForces  'Sets applied forces based on user selection
    freebodySet.SumContributions(1) = reactionForces  'Sets reaction forces based on user selection
    freebodySet.SumContributions(2) = multipointForces 'Sets multipoint contribution forces based on user selection
    freebodySet.AlternateApplied = appliedAltVect  'Sets alternate vectors for applied forces
    freebodySet.AlternateSPC = reactionAltVect  'Sets alternate vectors for reaction forces
    freebodySet.AlternateMPC = multipointAltVect  'Sets alternate vectors for multipoint constraint forces

    '<Prepares the excel objects if user prompted to send results to excel sheet>
    'Done after all groups and output sets have been verified, to avoid crashes and floating invisible Excel apps
    If toExcel = True Then
    	Set appExcel = New Excel.Application
    	Set wbk = appExcel.Workbooks.Add
    	For  i = wbk.Sheets.Count To  2 Step -1 'Go through each auto generated sheet and delete so that we start with one sheet
    		wbk.Sheets(wbk.Sheets(i).Name).Delete
    	Next
    End If

    '<Loops through groups to find summation forces>
    gID = gSet.First

    '<START LOOP>'
    While gID > 0

    	groups.Get(gID)
    	If toExcel = True Then
    		numSheets = numSheets + 1
    		If numSheets > 1 Then 'More than the original one sheet has been used, start creating new ones
    			wbk.Sheets.Add(After:=wbk.Sheets(wbk.Sheets.Count))
    			Set wks = wbk.Worksheets(numSheets)
    		Else
    			Set wks = wbk.Worksheets(numSheets)
    		End If
    		wks.Name = groups.title 'Sets the title of the worksheet
    		wks.Cells(row,col) = groups.title 'Sets the title in cell A1
    		With wks.Cells(row,col)
    			.Borders.LineStyle = xlContinuous 'Puts a border around cell A1
    			.Borders.Weight =xlMedium 'Thickens the border lines around cell A1
    			.HorizontalAlignment = xlCenter 'Centers the title in cell A1
    			.Font.Bold = True 'Bolds the font of the tile in cell A1
    		End With
    		wks.Cells(1,2) = numSheets 'Reference to number for sorting
    		row = row + 2 'Increments rows to start headers
    	End If

    	'Gets current group to retrieve elements, nodes, and coordinate systems
    	groups.Get(gID)
    	'Sets elements of current group for freebody
    	Set eSet = groups.List(FGR_ELEM)
    	freebodySet.SetElements(True, eSet.ID)

    	'Sets nodes of current group for freebody
    	Set nSet = groups.List(FGR_NODE)
    	freebodySet.SetNodes(True, nSet.ID)

    	'Checks for coordinate system(s) in group
    	If groupCoordSys = True Then
    		cSysExist = False
    		Set cSet = groups.List(FGR_CSYS)
    		If cSet Is Nothing Then 'If no coordinate systems exist in the group
    			App.feAppMessage(FCM_HIGHLIGHT, groups.title & " has no coordinate systems. Using Global coordinate system...")
    		Else
    			cSysExist =True 'At least one coordinate system exists in the group
    			cID = cSet.First 'Starts at first coordinate system
    			cID = cSet.Next 'Goes to potential second coordinate system
    			If cID > 0 Then 'If a second coordinate system exists
    				App.feAppMessage(FCM_HIGHLIGHT, groups.title & " has more than one coordinate system. Using first coordinate sysem...")
    			End If
    			cID = cSet.First 'Resets the cID back to the first coordinate system
    			freebodySet.CSys = cID
    		End If
    	End If

		'Calculate summation point for resultant vectors
		freebodySet.CalculateNodalCenter(True, averageNodalCenter)

    	'Calculates for all output sets
    	freebodySet.CalculateSummation2(oSet.ID, cSysExist, numSuccessOutputs, IDSuccessOutputs, calculationsOnOutputSets)
		Dim numNodes As Long
		Dim nodeIDs As Variant
		Dim nodalSums As Variant
        freebodySet.GetNodalSums(oSet.ID, numNodes, nodeIDs, nodalSums)

    	If toExcel = True Then 'Adds headers to the current excel sheet
    			wks.Cells(row,col) = "Output Set"
    			col = col + 1
    			wks.Cells(row, col) = "Fx"
    			col = col + 1
    			wks.Cells(row, col) = "Fy"
    			col = col + 1
    			wks.Cells(row, col) = "Fz"
    			col = col + 1
    			wks.Cells(row, col) = "Mx"
    			col = col + 1
    			wks.Cells(row, col) = "My"
    			col = col + 1
    			wks.Cells(row, col) = "Mz"
    			col = 1
    			row = row + 1
    	End If

    	'Gets results for each of the output sets
		For i = 1 To numNodes
			Fx_Val = nodalSums( 0 + (6*(i-1)) )
			If toMessageBoard = True Then
    			App.feAppMessage(FCM_NORMAL, groups.title & ", " & outputSets.title) 'Title of group and output set
    			App.feAppMessage(FCM_NORMAL, "Fx: " & vbTab & Fx_Val & vbTab & "Fy: " & vbTab & Fy_Val & vbTab & "Fz: " & vbTab & Fz_Val)
    			App.feAppMessage(FCM_NORMAL, "Mx: " & vbTab & Mx_Val & vbTab & "My: " & vbTab & My_Val & vbTab & "Mz: " & vbTab & Mz_Val)
    			App.feAppMessage(FCM_NORMAL, " ") 'Empty line for formatting
    		End If
		Next

    	For i = 1 To numSuccessOutputs
    		oID = IDSuccessOutputs(i-1)
    		outputSets.Get(oID)
    		'Results for each output set are stored next to each other in single array. New output set results every six places in array
    		Fx_Val = calculationsOnOutputSets( 0 + (6*(i-1)) )
    		Fy_Val = calculationsOnOutputSets( 1 + (6*(i-1)) )
    		Fz_Val = calculationsOnOutputSets( 2 + (6*(i-1)) )
    		Mx_Val = calculationsOnOutputSets( 3 + (6*(i-1)) )
    		My_Val = calculationsOnOutputSets( 4 + (6*(i-1)) )
    		Mz_Val = calculationsOnOutputSets( 5 + (6*(i-1)) )

    		If toMessageBoard = True Then
    			App.feAppMessage(FCM_NORMAL, groups.title & ", " & outputSets.title) 'Title of group and output set
    			App.feAppMessage(FCM_NORMAL, "Fx: " & vbTab & Fx_Val & vbTab & "Fy: " & vbTab & Fy_Val & vbTab & "Fz: " & vbTab & Fz_Val)
    			App.feAppMessage(FCM_NORMAL, "Mx: " & vbTab & Mx_Val & vbTab & "My: " & vbTab & My_Val & vbTab & "Mz: " & vbTab & Mz_Val)
    			App.feAppMessage(FCM_NORMAL, " ") 'Empty line for formatting
    		End If

    		If toExcel = True Then
    			'Puts information into excel cells
    			wks.Cells(row, col) = outputSets.title
    			col = col + 1
    			wks.Cells(row, col) = Fx_Val
    			col = col + 1
    			wks.Cells(row, col) = Fy_Val
    			col = col + 1
    			wks.Cells(row, col) = Fz_Val
    			col = col + 1
    			wks.Cells(row, col) = Mx_Val
    			col = col + 1
    			wks.Cells(row, col) = My_Val
    			col = col + 1
    			wks.Cells(row, col) = Mz_Val
    			col = 1
    			row = row + 1

    			'Puts borders around information
    			With wks.Range(wks.Cells(3, 1), wks.Cells(3+numSuccessOutputs, 7))
    				.Borders.LineStyle = xlContinuous
    			End With
    			With wks.Range(wks.Cells(3, 1), wks.Cells(3+numSuccessOutputs, 1))
    				.Borders.Weight = xlMedium
    			End With
    			With wks.Range(wks.Cells(3, 1), wks.Cells(3, 7))
    				.Borders.Weight = xlMedium
    			End With

    		End If
    	Next

    	'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    	'Resets row and column positions for next sheets
    	If toExcel = True Then
    		row = 1
    		col =1
    	End If
    	'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

    	gID = gSet.Next
    Wend
    '<END LOOP>'

    App.feAppMessage(FCM_HIGHLIGHT, "Task completed")

	''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    'Delete extraneous sheets
    If toExcel = True Then
    	'Autofits the columns of the remaining sheets
    	For i = 1 To numSheets
    		wbk.Worksheets(i).Columns("A:G").AutoFit
    	Next

   		'Makes the excel sheet visible
    	appExcel.Visible = True
    End If
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

    'If user did not select to save the freebody object, it is deleted
    If keepFreebody = False Then
    	If freebodySet.Deletable(fID) = True Then
    		freebodySet.Delete(fID)
    	End If
    Else
    	freebodySet.Get(fID)
    	freebodySet.Put(fID)
    	freebodySet.Active = fID
    End If

End Sub
